@@ -30,11 +30,7 @@ Please see the docstring on the backend module for more information, including
 notes on naming conventions.
 """
 
-try:
-    set
-except NameError:
-    from sets import Set as set     # Python 2.3 fallback
-
+import ldap
 import logging
 import pprint
 
@@ -56,7 +52,6 @@ class _LDAPConfig(object):
         of python-ldap.
         """
         if cls.ldap is None:
-            import ldap
             import ldap.filter
 
             # Support for python-ldap < 2.0.6
@@ -154,11 +149,12 @@ class LDAPSearch(object):
         try:
             filterstr = self.filterstr % filterargs
             results = connection.search_s(self.base_dn.encode('utf-8'),
-                self.scope, filterstr.encode('utf-8'))
-        except self.ldap.LDAPError, e:
+                                          self.scope,
+                                          filterstr.encode('utf-8'))
+        except ldap.LDAPError, e:
             results = []
             logger.error(u"search_s('%s', %d, '%s') raised %s" %
-                (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
+                         (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
 
         return self._process_results(results)
 
@@ -170,11 +166,11 @@ class LDAPSearch(object):
         try:
             filterstr = self.filterstr % filterargs
             msgid = connection.search(self.base_dn.encode('utf-8'),
-                self.scope, filterstr.encode('utf-8'))
-        except self.ldap.LDAPError, e:
+                                      self.scope, filterstr.encode('utf-8'))
+        except ldap.LDAPError, e:
             msgid = None
             logger.error(u"search('%s', %d, '%s') raised %s" %
-                (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
+                         (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
 
         return msgid
 
@@ -184,9 +180,9 @@ class LDAPSearch(object):
         """
         try:
             kind, results = connection.result(msgid)
-            if kind != self.ldap.RES_SEARCH_RESULT:
+            if kind != ldap.RES_SEARCH_RESULT:
                 results = []
-        except self.ldap.LDAPError, e:
+        except ldap.LDAPError, e:
             results = []
             logger.error(u"result(%d) raised %s" % (msgid, pprint.pformat(e)))
 
@@ -205,7 +201,8 @@ class LDAPSearch(object):
 
         result_dns = [result[0] for result in results]
         logger.debug(u"search_s('%s', %d, '%s') returned %d objects: %s" %
-            (self.base_dn, self.scope, self.filterstr, len(result_dns), "; ".join(result_dns)))
+                     (self.base_dn, self.scope, self.filterstr, len(result_dns),
+                      "; ".join(result_dns)))
 
         return results
 
@@ -349,12 +346,17 @@ class PosixGroupType(LDAPGroupType):
 
         try:
             user_uid = ldap_user.attrs['uid'][0]
-            user_gid = ldap_user.attrs['gidNumber'][0]
 
-            filterstr = u'(|(gidNumber=%s)(memberUid=%s))' % (
-                self.ldap.filter.escape_filter_chars(user_gid),
-                self.ldap.filter.escape_filter_chars(user_uid)
-            )
+            if 'gidNumber' in ldap_user.attrs:
+                user_gid = ldap_user.attrs['gidNumber'][0]
+                filterstr = u'(|(gidNumber=%s)(memberUid=%s))' % (
+                    self.ldap.filter.escape_filter_chars(user_gid),
+                    self.ldap.filter.escape_filter_chars(user_uid)
+                )
+            else:
+                filterstr = u'(memberUid=%s)' % (
+                    self.ldap.filter.escape_filter_chars(user_uid),
+                )
 
             search = group_search.search_with_additional_term_string(filterstr)
             groups = search.execute(ldap_user.connection)
@@ -370,17 +372,17 @@ class PosixGroupType(LDAPGroupType):
         """
         try:
             user_uid = ldap_user.attrs['uid'][0]
-            user_gid = ldap_user.attrs['gidNumber'][0]
 
             try:
                 is_member = ldap_user.connection.compare_s(group_dn.encode('utf-8'), 'memberUid', user_uid.encode('utf-8'))
-            except self.ldap.NO_SUCH_ATTRIBUTE:
+            except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                 is_member = False
 
             if not is_member:
                 try:
+                    user_gid = ldap_user.attrs['gidNumber'][0]
                     is_member = ldap_user.connection.compare_s(group_dn.encode('utf-8'), 'gidNumber', user_gid.encode('utf-8'))
-                except self.ldap.NO_SUCH_ATTRIBUTE:
+                except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                     is_member = False
         except (KeyError, IndexError):
             is_member = False
@@ -414,7 +416,7 @@ class MemberDNGroupType(LDAPGroupType):
                 self.member_attr.encode('utf-8'),
                 ldap_user.dn.encode('utf-8')
             )
-        except self.ldap.NO_SUCH_ATTRIBUTE:
+        except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
             result = 0
 
         return result
@@ -442,13 +444,14 @@ class NestedMemberDNGroupType(LDAPGroupType):
         those groups belong to, etc. Circular references will be detected and
         pruned.
         """
-        group_info_map = {} # Maps group_dn to group_info of groups we've found
-        member_dn_set = set([ldap_user.dn]) # Member DNs to search with next
-        handled_dn_set = set() # Member DNs that we've already searched with
+        group_info_map = {}  # Maps group_dn to group_info of groups we've found
+        member_dn_set = set([ldap_user.dn])  # Member DNs to search with next
+        handled_dn_set = set()  # Member DNs that we've already searched with
 
         while len(member_dn_set) > 0:
             group_infos = self.find_groups_with_any_member(member_dn_set,
-                group_search, ldap_user.connection)
+                                                           group_search,
+                                                           ldap_user.connection)
             new_group_info_map = dict([(info[0], info) for info in group_infos])
             group_info_map.update(new_group_info_map)
             handled_dn_set.update(member_dn_set)
@@ -520,3 +523,20 @@ class NestedActiveDirectoryGroupType(NestedMemberDNGroupType):
     """
     def __init__(self, name_attr='cn'):
         super(NestedActiveDirectoryGroupType, self).__init__('member', name_attr)
+
+
+class OrganizationalRoleGroupType(MemberDNGroupType):
+    """
+    An LDAPGroupType subclass that handles groups of class organizationalRole.
+    """
+    def __init__(self, name_attr='cn'):
+        super(OrganizationalRoleGroupType, self).__init__('roleOccupant', name_attr)
+
+
+class NestedOrganizationalRoleGroupType(NestedMemberDNGroupType):
+    """
+    An LDAPGroupType subclass that handles groups of class OrganizationalRoleGroupType
+    with nested group references.
+    """
+    def __init__(self, name_attr='cn'):
+        super(NestedOrganizationalRoleGroupType, self).__init__('roleOccupant', name_attr)
